<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class BarLine extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_BARLINE = "5898c8f3ca6bc3c1f61960d9";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getXAxis(){
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            $categoria[] = $row[$widget['chartXAxisField']];
        }
        $xAxis = [
            "mask" => !empty($widget['chartXAxisFormat']) ? $widget['chartXAxisFormat'] : null,
            "categories" => $categoria,
            "field" => $widget['chartXAxisField'],
            "labels" => [
                "show" => true,
                "rotate" => !empty($widget['chartXAxisTicksRotation']) ? $widget['chartXAxisTicksRotation']-90 : -60,
            ],
            "tickPlacement" => "between"
        ];

        if($xAxis["labels"]["rotate"] != 0){
            $xAxis["labels"]["rotateAlways"] = false;
        }

        return $xAxis;
    }

    protected function getYAxis(){
        $widget = $this->widget;
        $series = $this->getSeries();
        $serieTypeIndex = array_flip(array_column($series,"type"));
        $seriesNameIndex = [$series[$serieTypeIndex["column"]]["name"], $series[$serieTypeIndex["line"]]["name"]];
        $serieTypeShow = [true, true];
        $maskIndex = ["serieYAxisFormatBar", "serieYAxisFormatLine"];
        $yAxisArray = [];
        $i = 0;
        foreach($series as $serie){
            $i = $serie["type"] == "column" ? 0 : 1;
            $yAxis = [
                "show" => $serieTypeShow[$i],
                "mask" => !empty($widget[$maskIndex[$i]]) ? $widget[$maskIndex[$i]] : null,
                "dataLabels" => [],
                "seriesName" => $seriesNameIndex[$i],
                "labels" => [
                    "show" => true,
                ],
                "opposite" => $i==1,
                "axisTicks" => [
                    "show" => true
                ]
            ];

            if(!empty($widget['serieYAxisLabel'])){
                $yAxis['title'] = [
                    "text" => $widget['serieYAxisLabel'],
                    "style" => [
                        "fontSize" => "16px"
                    ]
                ];
            }
            $serieTypeShow[$i] = false;
            $yAxisArray[] = $yAxis;
        }

        return $yAxisArray;
    }

    protected function getSeries(){
        $widget = $this->widget;
        $series = [];
            foreach ($widget['chartSeries'] as $serieIndex => $serie) {
                foreach($serie["serieFields"] as $index => $fact){
                        $serieType = $serie["serieType"];
                        if($serieType == "bar"){
                            $serieType = "column";
                        }
                        $series[] = array(
                            "name" => $serie['serieLabels'][$index] ,
                            "field" => $fact,
                            "type" => $serieType,
                            "data" => array_column($this->widgetData, $fact),
                        );
                }
            }
        return $series;
    }

    protected function getTooltip(){
        $widget = $this->widget;
        return [
            "enabled" => true
        ];
    }

    protected function getPlotOptions(){
        $widget = $this->widget;
        $barOptions = [
            "horizontal" => false,
            "dataLabels" => [
                "position" => !empty($widget['chartInvertedAxes']) ? 'left' : 'top',
                "textAnchor" => "end",
            ],
        ];

        if(!empty($widget['chartXAxisBandDimensionGap'])){
            $barOptions["barHeight"] = $this->getBarHeightByDimensionGap($widget['chartXAxisBandDimensionGap']);
        }

        return [
            "bar" => $barOptions
        ];
    }

    protected function getBarHeightByDimensionGap($dimensionGap){
        return strval((1-$dimensionGap)*100 )."%";
    }

    protected function getSerieStacked(){
        $widget = $this->widget;

        if(!empty($widget['serieStackedBar'])){
            return $widget['serieStackedBar'];
        }
        return false;
    }

    protected function getChart(){
        $widget = $this->widget;

        return [
            "type" => "line",
            "id" => $widget['id'],
            "stacked" => $this->getSerieStacked(),
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors(){
        $widget = $this->widget;
        if(!empty($widget["chartSeries"])){
            $colors = [];
            foreach($widget["chartSeries"] as $chartSerie){
                if(!empty($chartSerie["serieColors"])){
                    foreach($chartSerie["serieColors"] as $color){
                        $colors[] = $color;
                    }
                }
            }
            return $colors;
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        $enabled = $widget["serieShowValueLabelsBar"] || $widget["serieShowValueLabelsLine"];
        $enabledOnSeries = [];
        if($enabled){
            foreach($this->getSeries() as $i => $serie){
                if(($serie["type"] == "line" && $widget["serieShowValueLabelsLine"]) || ($serie["type"] == "column" && $widget["serieShowValueLabelsBar"])){
                    $enabledOnSeries[] = $i;
                }
            }
        }
        return [
            "enabled" => $enabled,
            "enabledOnSeries" => $enabledOnSeries,
            "position" => "top",
            "textAnchor" => "middle",
            "offsetY" => -8,
            "style" => [
                "colors" => $this->getSerieStacked() ? [] : $this->getSerieColors()
            ],
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;
        return [
            "text" => $widget['name'],
            "style" => [
                "fontSize" => "16px"
            ]
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
              "fontSize" => '14px',
            ],
        ];
    }

    protected function getLineMarkerShape(){
        $widget = $this->widget;
        if(!empty($widget["serieLineMarkTypeLine"])){
            switch($widget["serieLineMarkTypeLine"]){
                default: case "circle": case "cross":
                    return "circle";
                case "square": case "diamond": case "triangle-up": case "triangle-down":
                    return "square";
            }
        }
        return "circle";
    }

    protected function getOptions(){
        $widget = $this->widget;

        $legendPosition = "top";
        if(!empty($widget["chartLegendPosition"])){
            $legendPosition = explode(" ",$widget["chartLegendPosition"])[0];
        }

        return [
            "colors" => $this->getSerieColors(),
            "drillDownView" => $widget['drillDownView'],
            "plotOptions" => $this->getPlotOptions(),
            "dataLabels" => $this->getDataLabels(),
            "chart" => $this->getChart(),
            "legend" => [
                "position" => $legendPosition
            ],
            "zoom" > [
                "enabled" => true
            ],
            "markers" => [
                "size" => isset($widget["serieLineMarkTypeLine"]) ? 8 : 0,
                "shape" => $this->getLineMarkerShape(),
            ],
            "title" => $this->getTitle(),
            "noData" => $this->getNoData(),
            "tooltip" => $this->getTooltip(),
            "xaxis" => $this->getXAxis(),
            "yaxis" => $this->getYAxis(),
            "stroke" => [
                "show" => true,
            ],
        ];
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_BARLINE,
            "component" => self::CHART_COMPONENT,
            "chartType" => "line",
            "height" =>"100%",
            // bug versão do next
            "style" =>"height: 99%;",
            "options" => $this->getOptions(),
            "series" => $this->getSeries(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
